<?php
/**
 * VuFind Cache Manager
 *
 * PHP version 5
 *
 * Copyright (C) Villanova University 2007.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @category VuFind2
 * @package  Cache
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     http://www.vufind.org  Main Page
 */
namespace VuFind\Cache;
use VuFind\Config\Reader as ConfigReader, Zend\Cache\StorageFactory;

/**
 * VuFind Cache Manager
 *
 * Creates file and APC caches
 *
 * @category VuFind2
 * @package  Cache
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     http://www.vufind.org  Main Page
 */
class Manager
{
    /**
     * Default configuration settings.
     *
     * @var array
     */
    protected $defaults;

    /**
     * Was there a problem building cache directories?
     *
     * @var bool
     */
    protected $directoryCreationError = false;

    /**
     * Settings used to generate cache objects.
     *
     * @var array
     */
    protected $cacheSettings = array();

    /**
     * Actual cache objects generated from settings.
     *
     * @var array
     */
    protected $caches = array();

    /**
     * Constructor
     */
    public function __construct()
    {
        // Get global cache options
        $config = ConfigReader::getConfig('config');

        // $config and $config->Cache are Zend\Config\Config objects
        // $cache is created immutable, so get the array, it will be modified
        // downstream.
        // Zend\Config\Config can be created mutable or cloned and merged, useful
        // for future cache-specific overrides.
        $cacheConfig = isset($config->Cache) ? $config->Cache : false;
        $this->defaults = $cacheConfig ? $cacheConfig->toArray() : false;

        // Get base cache directory.
        $cacheBase = $this->getCacheDir();

        // Set up basic object cache:
        $this->createFileCache('object', $cacheBase . 'objects');

        // Set up language cache:
        $this->createFileCache('language', $cacheBase . 'languages');

        // Set up search specs cache based on config settings:
        $config = ConfigReader::getConfig('searches');
        $searchCacheType = isset($config->Cache->type)
            ? $config->Cache->type : false;
        switch ($searchCacheType) {
        case 'APC':
            $this->createAPCCache('searchspecs');
            break;
        case 'File':
            $this->createFileCache(
                'searchspecs', $cacheBase . 'searchspecs'
            );
            break;
        }
    }

    /**
     * Retrieve the specified cache object.
     *
     * @param string $key Key identifying the requested cache.
     *
     * @return object
     */
    public function getCache($key)
    {
        if (!isset($this->caches[$key])) {
            if (!isset($this->cacheSettings[$key])) {
                throw new \Exception('Requested unknown cache: ' . $key);
            }
            $this->caches[$key] = StorageFactory::factory(
                $this->cacheSettings[$key]
            );
        }
        return $this->caches[$key];
    }

    /**
     * Get the path to the directory containing VuFind's cache data.
     *
     * @return string
     */
    public function getCacheDir()
    {
        if ($this->defaults && isset($this->defaults['cache_dir'])) {
            $dir = $this->defaults['cache_dir'];
            // ensure trailing slash:
            if (substr($dir, -1) != '/') {
                $dir .= '/';
            }
            return $dir;
        }
        if (strlen(LOCAL_OVERRIDE_DIR) > 0) {
            return LOCAL_OVERRIDE_DIR . '/cache/';
        }
        return APPLICATION_PATH . '/data/cache/';
    }

    /**
     * Check if there have been problems creating directories.
     *
     * @return bool
     */
    public function hasDirectoryCreationError()
    {
        return $this->directoryCreationError;
    }

    /**
     * Add a file cache to the manager and ensure that necessary directory exists.
     *
     * @param string $cacheName Name of new cache to create
     * @param string $dirName   Directory to use for storage
     *
     * @return void
     */
    protected function createFileCache($cacheName, $dirName)
    {
        $opts = $this->defaults;    // copy defaults -- we'll modify them below
        if (!is_dir($dirName)) {
            if (isset($opts['umask'])) {
                // convert umask from string
                $umask = octdec($opts['umask']);
                // validate
                if ($umask & 0700) {
                    throw new \Exception(
                        'Invalid umask: ' . $opts['umask']
                        . '; need permission to execute, read and write by owner'
                    );
                }
                umask($umask);
            }
            if (isset($opts['dir_permission'])) {
                $dir_perm = octdec($opts['dir_permission']);
            } else {
                // 0777 is chmod default, use if dir_permission is not explicitly set
                $dir_perm = 0777;
            }
            if (!@mkdir($dirName, $dir_perm)) {
                $this->directoryCreationError = true;
            }
        }
        if (empty($opts)) {
            $opts = array('cache_dir' => $dirName);
        } elseif (is_array($opts)) {
            // If cache_dir was set in config.ini, the cache-specific name should
            // have been appended to the path to create the value $dirName.
            $opts['cache_dir'] = $dirName;
        } else {
            // Dryrot
            throw new \Exception('$opts is neither array nor false');
        }
        $this->cacheSettings[$cacheName] = array(
            'adapter' => array('name' => 'filesystem', 'options' => $opts),
            'plugins' => array('serializer')
        );
    }

    /**
     * Add an APC cache to the manager.
     *
     * @param string $cacheName Name of new cache to create
     *
     * @return void
     */
    protected function createAPCCache($cacheName)
    {
        $this->cacheSettings[$cacheName] = array(
            'adapter' => 'APC',
            'plugins' => array('serializer')
        );
    }
}